# This is the code malthus.m from Jim Powell's webpage implemented in R.
# (see http://www.math.usu.edu/powell/wauclass/labs.html)
# This is a 1 dimensional integrodifference equation solved using the
# discrete fourier transform

## FUNCTIONS
# In matlab you can define functions at the end of your code, but in
# R you must define them before use. 

# The function ffshift in R from the mrbsizeR package appears to only work for
# matrix inputs so I coded a version of this function that will work for
# vectors since our space is 1 dimensional.
fftshift1=function(x){
  x1 = ceiling(length(x)/2);
  y = c(x[(x1+1):length(x)],x[1:x1]);
  return(y)
}

ifftshift1=function(x){
  x1 = floor(length(x)/2);
  y = c(x[(x1+1):length(x)],x[1:x1]);
  return(y)
}
#################

## MALTHUS
#       p(x)    -       population density in x
#       K(x)    -       probability of individual dispersal to location
#                          x starting at location 0.
#       xl	-       maximum x coordinate (also - minimum)
#       dx	-       spatial step  delta x = 2*xl/np
#       np	-       number of points  in discretization for x
#       		   (should be a power of 2 for  efficiency of FFT)
#       c	-       center of spatial  dispersal
#       D	-       diffusion constant (per step)
#       r	-       per-capita growth constant

np = 128;
xl=5;
dx = 2*xl/np;
c=0.4;
D=0.25
r=1.15;
nsteps=10;

#       discretize space, define an initial population distribution
x = -xl+seq(0,np-1)*dx;
p = (abs(x+3)<=.5);
p0=p;

#       define a dispersal kernel
K = exp(-(x-c)^2/(4*D))/(4*pi*D)^.5;

#       normalize K to make it have integral one
K=K/(dx*sum(K));

#       calculate the fft of K, multiplying by dx to account
#       for the additional factor of np and converting from a
#       interval length of 1 to 2*xl.  The fftshift accounts for
#       using an interval of (-xl,xl) as opposed to (0,2*xl).
fK = fft(fftshift1(K))*dx

# plot the inital distribution in read
plot(x,p0,typ="l", col = "red", xlim = c(-xl,xl), ylim = c(0, 1.5), xlab = "space", ylab="population density")

#       begin iterating the model 
for(i in 1:nsteps){
  #       calculate the fft of p; perform the convolution
  fp = fft(p);
  fg = fp*fK;
  #       fg now contains the fourier transform of the convolution;
  #       invert it, multiply by post-dispersal reproduction,
  #       and take a look. Unlike MATLAB, R returns and unnormalized
  #       ifft so for equivalent results, I divide by length(x).
  g = r*abs(fft(fg, inverse=TRUE)/length(x));
  #       update p and move to the next time step
  p=g;
  lines(x,g,typ="l", col="blue")
}


